SystemOrganization addCategory: #KomAJPServer!

HttpResponse subclass: #AJPResponse
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KomAJPServer'!

!AJPResponse commentStamp: 'cs 6/15/2009 20:51' prior: 0!
An AJPResponse is the equivalent to the HttpResponse. It can write itself onto an AJPStream.

Instance Variables
!

!AJPResponse class methodsFor: 'instance creation' stamp: 'cs 6/15/2009 16:02'!
newFrom: aHttpResponse
	"Create a new instance from a given HttpResponse"

	^ self new
		status: aHttpResponse status;
		contents: aHttpResponse contents;
		contentType: aHttpResponse contentType;
		fields: aHttpResponse fields;
		addCookies: aHttpResponse cookies;
		yourself! !

!AJPResponse class methodsFor: 'preferences' stamp: 'cs 6/15/2009 17:38'!
responseChunkSize

	^ 8192! !

!AJPResponse methodsFor: 'accessing' stamp: 'cs 6/15/2009 16:01'!
fields: aDictionary
	"Missing setter for the fields dictionary"

	fields := aDictionary.! !

!AJPResponse methodsFor: 'responding' stamp: 'cs 6/15/2009 19:30'!
writeHeadersOn: aStream
	"Writes a headers packet"
	
	aStream
		beginPacket;
		nextPutInteger: self statusCode asInteger;
		nextPutString: (StatusCodes at: self status) value.
	self writeResponseHeadersOn: aStream.
	aStream endHeader.! !

!AJPResponse methodsFor: 'responding' stamp: 'cs 6/15/2009 20:09'!
writeOn: anAJPStream 
	"Writes this response onto the provided AJPStream."
	super writeOn: anAJPStream.
	anAJPStream endResponse! !

!AJPResponse methodsFor: 'responding' stamp: 'cs 6/15/2009 19:05'!
writeResponseHeadersOn: aStream
	"Writes AJP-encoded headers on the provided stream"
	
	| headers |
	headers := OrderedCollection new.
	headers
		addAll: self fields associations;
		addAll: (self cookies collect: [:cookie | 'Set-Cookie' -> cookie]);
		add: 'Content-Type' -> self contentType;
		add: 'Content-Length' -> self contentLength.
	aStream nextPutInteger: headers size.
	headers do: [:assoc |
		aStream
			nextPutResponseHeader: assoc key asLowercase;
			nextPutString: assoc value asString].! !

Object subclass: #AJPConnection
	instanceVariableNames: 'target stream lastRequest'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KomAJPServer'!

!AJPConnection commentStamp: 'cs 6/15/2009 20:48' prior: 0!
An AJPConnection handles a TCP connection to the webserver. One connection can receive different kinds of AJP packets, processes these packets with the target web application and returns corresponding AJP packets back to the webserver.

Instance Variables
	lastRequest:		<HttpRequest>
	stream:		<AJPStream>
	target:		<Object>

lastRequest
	- The last request. Stored to implement the lazy loading of POST request contents.

stream
	- The AJPStream used to communicate with the webserver.

target
	- The target that actually processes the requests, e.g. a seaside application!

!AJPConnection methodsFor: 'testing' stamp: 'cs 6/14/2009 14:47'!
isWaitingForBody
	"Indicates that the next packet is expected as data packet (e.g. after POST)"
	
	^ self lastRequest notNil and: [self lastRequest isWaitingForBody]! !

!AJPConnection methodsFor: 'accessing' stamp: 'cs 6/14/2009 14:44'!
lastRequest
	"Returns the last AJPRequest. Needed for loading additional body data"

	^ lastRequest! !

!AJPConnection methodsFor: 'accessing' stamp: 'cs 6/14/2009 14:44'!
lastRequest: anObject
	"Sets the last AJPRequest. Needed for loading additional body data"

	lastRequest := anObject.! !

!AJPConnection methodsFor: 'processing' stamp: 'cs 6/15/2009 16:46'!
processCPingPacket
	"The web server asks the container to respond quickly with a CPong"! !

!AJPConnection methodsFor: 'processing' stamp: 'cs 6/15/2009 16:45'!
processDataPacket: length
	"Additional data for the body of the request (usally POST)"! !

!AJPConnection methodsFor: 'processing' stamp: 'cs 6/15/2009 18:21'!
processForwardRequestPacket
	"Processes a normal forward request"
	
	| response |
	self lastRequest: (AJPRequest readFromStream: self stream).
	response := AJPResponse newFrom: self processHttpRequest.
	self lastRequest method = 'HEAD'
		ifTrue: [response writeHeadersOn: self stream]
		ifFalse: [response writeOn: self stream].! !

!AJPConnection methodsFor: 'processing' stamp: 'cs 6/14/2009 14:47'!
processHttpRequest
	"Delegate the actual processing to the target"
	
	^ self target processHttpRequest: self lastRequest! !

!AJPConnection methodsFor: 'processing' stamp: 'cs 6/15/2009 16:44'!
processNormalPacket
	"Process depending on the packet type"
	
	self perform: self stream nextPacketType.! !

!AJPConnection methodsFor: 'processing' stamp: 'cs 6/13/2009 23:00'!
processPingPacket
	"The web server asks the container to take control (secure login phase)"! !

!AJPConnection methodsFor: 'processing' stamp: 'cs 6/13/2009 22:27'!
processShutdownPacket
	"Stops the AJP server"
	
	self stream socket remoteAddress = self stream socket localAddress
		ifFalse: [^ self]. "only localhost may shutdown the server for security reasons"

	self target stop.
	self stream close.! !

!AJPConnection methodsFor: 'serving' stamp: 'cs 6/15/2009 16:44'!
readPacket: length 
	"Reads the incoming packet to the provided length and processed the packet"
	
	self isWaitingForBody 
		ifTrue: [self processDataPacket: length]
		ifFalse: [self processNormalPacket]! !

!AJPConnection methodsFor: 'serving' stamp: 'cs 6/15/2009 16:43'!
run
	"Runs the read-process cycle until the connection is closed"

	[self stream atEnd] whileFalse: [
		self
			verifyMagic;
			readPacket: self stream nextInteger].! !

!AJPConnection methodsFor: 'accessing' stamp: 'cs 6/14/2009 14:43'!
stream
	"Returns the network stream for the connection"

	^ stream! !

!AJPConnection methodsFor: 'accessing' stamp: 'cs 6/15/2009 16:30'!
stream: aStream
	"Sets the network stream for the connection"

	stream := AJPStream on: aStream.! !

!AJPConnection methodsFor: 'accessing' stamp: 'cs 6/14/2009 12:59'!
target
	"Returns the actual target that processes HttpRequests and answers HttpResponses"

	^ target! !

!AJPConnection methodsFor: 'accessing' stamp: 'cs 6/14/2009 12:59'!
target: anObject
	"Sets the actual target that processes HttpRequests and answers HttpResponses"

	target := anObject.! !

!AJPConnection methodsFor: 'serving' stamp: 'cs 6/15/2009 16:43'!
verifyMagic
	"Every AJP packet from the server to the container begins with 0x1234"

	(self stream nextByte = 18 and: [self stream nextByte = 52])
		ifFalse: [self error: 'Invalid AJP request'].! !

HttpRequest subclass: #AJPRequest
	instanceVariableNames: 'isSSL attributes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KomAJPServer'!

!AJPRequest commentStamp: 'cs 6/15/2009 20:50' prior: 0!
An AJPRequest is the equivalent to the HttpRequest with some special features. The contents of a request are parsed from a AJPStream.

Instance Variables
	attributes:		<Dictionary>
	isSSL:		<Boolean>

attributes
	- a collection of key-value-pairs received from the webserver.

isSSL
	- whether the requests used SSL!

!AJPRequest methodsFor: 'attributes' stamp: 'cs 6/15/2009 20:40'!
attributeAt: aSymbol put: anObject
	"Adds the provided objects under the provided attribute key"

	aSymbol = #queryString ifTrue:
		[^ self rawUrl: self url, '?', anObject].
	self attributes at: aSymbol put: anObject.! !

!AJPRequest methodsFor: 'accessing' stamp: 'cs 6/14/2009 14:52'!
attributes
	"Returns additional attributes of AJP"

	^ attributes! !

!AJPRequest methodsFor: 'accessing' stamp: 'cs 6/14/2009 14:53'!
attributes: anObject
	"Sets additional attributes of AJP"

	attributes := anObject.! !

!AJPRequest methodsFor: 'initialize-release' stamp: 'cs 6/15/2009 20:16'!
initialize
	"Initializes the attribute dictionary"

	super initialize.
	self attributes: Dictionary new.! !

!AJPRequest methodsFor: 'accessing' stamp: 'cs 6/14/2009 14:53'!
isSSL
	"Returns whether this request used SSL"

	^ isSSL! !

!AJPRequest methodsFor: 'accessing' stamp: 'cs 6/14/2009 14:53'!
isSSL: anObject
	"Sets whether this requests used SSL"

	isSSL := anObject.! !

!AJPRequest methodsFor: 'accessing' stamp: 'cs 6/14/2009 14:54'!
isWaitingForBody
	"Returns true if the POST request has not loaded all body data yet"

	^ self isPostRequest and:
		[(self propertyAt: #rawPostFields ifAbsent: [^ true])
			size < self contentLength]! !

!AJPRequest methodsFor: 'parsing' stamp: 'cs 6/15/2009 20:34'!
parseAttributes
	"Parses the attributes"

	| type |
	[type := self stream nextAttributeKey.
	type = #done ifTrue: [^ self]. "terminate on done-attribute"
	self attributeAt: type put: self stream nextString]
		repeat.! !

!AJPRequest methodsFor: 'parsing' stamp: 'cs 6/14/2009 14:50'!
parseHeaders
	"Parses the headers"

	| count |
	self header: Dictionary new.
	count := self stream nextInteger.
	count timesRepeat:
		[self header
			at: self stream nextHeaderKey
			put: self stream nextString].! !

!AJPRequest methodsFor: 'parsing' stamp: 'cs 6/14/2009 15:02'!
parseMethod
	"Parses method string, e.g. GET"

	self method: self stream nextMethodCode.! !

!AJPRequest methodsFor: 'parsing' stamp: 'cs 6/14/2009 14:50'!
parseProtocol
	"Parses the protocol string, e.g. HTTP/1.1"

	self protocol: self stream nextString.! !

!AJPRequest methodsFor: 'parsing' stamp: 'cs 6/14/2009 14:51'!
parseRemoteAddress
	"Parses the remote address, e.g. 127.0.0.1"

	self remoteAddress: self stream nextString.! !

!AJPRequest methodsFor: 'parsing' stamp: 'cs 6/14/2009 14:51'!
parseRemoteHost
	"Parses the remote host - not saved here"

	self stream nextString.! !

!AJPRequest methodsFor: 'parsing' stamp: 'cs 6/14/2009 14:49'!
parseRequest
	"Parses an AJP request packet"

	self
		parseMethod;
		parseProtocol;
		parseURI;
		parseRemoteAddress;
		parseRemoteHost;
		parseServerName;
		parseServerPort;
		parseSSL;
		parseHeaders;
		parseAttributes.! !

!AJPRequest methodsFor: 'parsing' stamp: 'cs 6/14/2009 14:51'!
parseSSL
	"Parses the isSSL flag"

	self isSSL: self stream nextBoolean.! !

!AJPRequest methodsFor: 'parsing' stamp: 'cs 6/14/2009 14:54'!
parseServerName
	"Parses the servername - not saved here"

	self stream nextString.! !

!AJPRequest methodsFor: 'parsing' stamp: 'cs 6/14/2009 14:55'!
parseServerPort
	"Parses the server port"

	self propertyAt: #localPort put: self stream nextInteger.! !

!AJPRequest methodsFor: 'parsing' stamp: 'cs 6/14/2009 14:55'!
parseURI
	"Parses the URI, e.g. /ajpe/helloworld"

	self rawUrl: self stream nextString.! !

!AJPRequest methodsFor: 'accessing' stamp: 'cs 6/14/2009 14:21'!
rawUrl: aString
	"Sets the rawUrl. This setter is missing in HttpRequest"
	
	rawUrl := aString.
	url := ((rawUrl copyUpTo: $?) unescapePercentsWithTextEncoding: nil). "remove query"
	rawUrl size - url size > 1 "if there is a query, set it"
		ifTrue: [self queryString: (rawUrl copyFrom: url size + 2 to: rawUrl size)].! !

!AJPRequest methodsFor: 'initialize-release' stamp: 'cs 6/14/2009 14:55'!
readRequestHeaderFrom: aStream
	"Overridden to enable custom AJP parsing from the stream."
	
	self parseRequest.! !

Stream subclass: #AJPStream
	instanceVariableNames: 'stream packetStream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KomAJPServer'!

AJPStream class
	instanceVariableNames: 'packetTypes methodCodes headerCodes attributeCodes responseHeaderCodes'!

!AJPStream commentStamp: 'cs 6/15/2009 20:55' prior: 0!
An AJPStream is a Stream that operates ontop of another Stream and is an abstraction layer for the AJP communication. Encoding of primitive and not-so-primitive types is done by the AJPStream as well as composing packets to the webserver.

Instance Variables
	packetStream:		<Stream>
	stream:		<Stream>

packetStream
	- This stream is used to store the contents of a packet that will be sent later. This is needed because the length of a packet must be send before the packet itself.

stream
	- This stream is the actual underlying stream, e.g. a network SocketStream
!

AJPStream class
	instanceVariableNames: 'packetTypes methodCodes headerCodes attributeCodes responseHeaderCodes'!

!AJPStream class methodsFor: 'mapping' stamp: 'cs 6/14/2009 15:19'!
attributeCode: aSmallInteger
	"Returns a string representation of the AJP attribute code"

	attributeCodes ifNil: [attributeCodes := Dictionary newFrom: {
		1 -> #context.
		2 -> #servletPath.
		3 -> #remoteUser.
		4 -> #authType.
		5 -> #queryString.
		6 -> #route.
		7 -> #sslCertificate.
		8 -> #sslCipher.
		9 -> #sslSession.
		10 -> #reqAttribute.
		11 -> #sslKeySize.
		12 -> #secret.
		13 -> #storedMethod.
		255 -> #done}].
	^ attributeCodes at: aSmallInteger ifAbsent: [self error: 'Invalid Attribute Code']! !

!AJPStream class methodsFor: 'mapping' stamp: 'cs 6/14/2009 15:55'!
headerCode: aSmallInteger
	"Returns a string representation of the AJP header code"

	headerCodes ifNil: [headerCodes := Dictionary newFrom: {
		1 -> 'accept'.
		2 -> 'accept-charset'.
		3 -> 'accept-encoding'.
		4 -> 'accept-language'.
		5 -> 'authorization'.
		6 -> 'connection'.
		7 -> 'content-type'.
		8 -> 'content-length'.
		9 -> 'cookie'.
		10 -> 'cookie2'.
		11 -> 'host'.
		12 -> 'pragma'.
		13 -> 'referer'.
		14 -> 'user-agent'}].
	^ headerCodes at: aSmallInteger ifAbsent: [self error: 'Invalid Header Code']! !

!AJPStream class methodsFor: 'mapping' stamp: 'cs 6/14/2009 15:03'!
methodCode: aSmallInteger
	"Returns a string representation of the AJP method code"

	methodCodes ifNil: [methodCodes := Dictionary newFrom: {
		1	-> 'OPTIONS'.
		2	-> 'GET'.
		3	-> 'HEAD'.
		4	-> 'POST'.
		5	-> 'PUT'.
		6	-> 'DELETE'.
		7	-> 'TRACE'.
		8	-> 'PROPFIND'.
		9	-> 'PROPPATCH'.
		10	-> 'MKCOL'.
		11	-> 'COPY'.
		12	-> 'MOVE'.
		13	-> 'LOCK'.
		14	-> 'UNLOCK'.
		15	-> 'ACL'.
		16	-> 'REPORT'.
		17	-> 'VERSION-CONTROL'.
		18	-> 'CHECKIN'.
		19	-> 'CHECKOUT'.
		20	-> 'UNCHECKOUT'.
		21	-> 'SEARCH'.
		22	-> 'MKWORKSPACE'.
		23	-> 'UPDATE'.
		24	-> 'LABEL'.
		25	-> 'MERGE'.
		26	-> 'BASELINE_CONTROL'.
		27	-> 'MKACTIVITY'}].
	^ methodCodes at: aSmallInteger ifAbsent: [self error: 'Invalid Method Code']! !

!AJPStream class methodsFor: 'instance creation' stamp: 'cs 6/15/2009 18:23'!
on: aStream
	
	^ self basicNew
		stream: aStream;
		yourself! !

!AJPStream class methodsFor: 'mapping' stamp: 'cs 6/15/2009 16:46'!
packetType: aSmallInteger
	"Returns the symbol for the processing message corresponding to the packet type"

	packetTypes ifNil: [packetTypes := Dictionary
		with: 2 -> #processForwardRequestPacket
		with: 7 -> #processShutdownPacket
		with: 8 -> #processPingPacket
		with: 10 -> #processCPingPacket].
	^ packetTypes at: aSmallInteger ifAbsent: [self error: 'Invalid Packet Type']! !

!AJPStream class methodsFor: 'mapping' stamp: 'cs 6/15/2009 18:39'!
responseHeaderCode: aString
	"Returns the special AJP code of the provided string; otherwise returns nil"

	responseHeaderCodes ifNil: [responseHeaderCodes := Dictionary newFrom: {
		'content-type' -> 1.
		'content-language' -> 2.
		'content-length' -> 3.
		'date' -> 4.
		'last-modified' -> 5.
		'location' -> 6.
		'set-cookie' -> 7.
		'set-cookie2' -> 8.
		'servlet-engine' -> 9.
		'status' -> 10.
		'www-authenticate' -> 11}].
	^ responseHeaderCodes at: aString ifAbsent: []! !

!AJPStream methodsFor: 'testing' stamp: 'cs 6/15/2009 16:34'!
atEnd
	"Answer whether the receiver can access any more objects."
	
	^ self stream atEnd! !

!AJPStream methodsFor: 'packets' stamp: 'cs 6/15/2009 17:14'!
beginPacket
	"Begins a new packet. All following puts will only affect the packet.
	A complete packet can be sent with endPacket"

	self packetStream: (WriteStream with: String new).! !

!AJPStream methodsFor: 'file open/close' stamp: 'cs 6/15/2009 20:41'!
close
	"Closes the underlying stream"

	self stream close.! !

!AJPStream methodsFor: 'testing' stamp: 'cs 6/15/2009 20:41'!
closed
	"Returns true if the underlying stream is closed"

	^ self stream closed! !

!AJPStream methodsFor: 'accessing' stamp: 'cs 6/15/2009 16:34'!
contents
	"Answer all of the contents of the receiver."
	
	^ self stream contents! !

!AJPStream methodsFor: 'packets' stamp: 'cs 6/15/2009 20:08'!
endBodyChunk
	"All data written to the packetStream will be send as AJP body chunk packet"

	| packet |
	packet := self endPacket.
	self
		nextPutInteger: packet size + 4;
		nextPutByte: 3 "AJP13_SEND_BODY_CHUNK";
		nextPutInteger: packet size;
		nextPutAll: packet contents;
		nextPutByte: 0.
	self flush.! !

!AJPStream methodsFor: 'packets' stamp: 'cs 6/15/2009 19:37'!
endHeader
	"All data written to the packetStream will be send as AJP header packet"
	
	| packet |
	packet := self endPacket.
	self
		nextPutInteger: packet size + 1;
		nextPutByte: 4; "AJP13_SEND_HEADERS"
		nextPutAll: packet contents.
	self flush.! !

!AJPStream methodsFor: 'packets' stamp: 'cs 6/15/2009 19:33'!
endPacket
	"Starts sending a complete packet to the underlying stream"
	
	| packet |
	packet := self packetStream.
	self packetStream: nil. "Resetting packet mode"
	self
		nextPutByte: 65; "Each packet from the container to the webserver starts with AB"
		nextPutByte: 66.
	^ packet! !

!AJPStream methodsFor: 'packets' stamp: 'cs 6/15/2009 20:11'!
endResponse
	"Sends an end response packet"
	
	self endPacket.
	self
		nextPutInteger: 2;
		nextPutByte: 5; "AJP13_END_RESPONSE"
		nextPutBoolean: true. "Reuse TCP connection"
	self flush.! !

!AJPStream methodsFor: 'packets' stamp: 'cs 6/15/2009 19:39'!
flush
	"If there is a pending packet then send it and create a new one;
	otherwise simply flush the underlying stream.
	This enabled sending body chunks over the stream with automatic packeting."

	self packetStream
		ifNil: [self stream flush]
		ifNotNil: [self endBodyChunk].
	self beginPacket.! !

!AJPStream methodsFor: 'initialize-release' stamp: 'cs 6/15/2009 19:22'!
initialize
	"Creates the first packet."

	super initialize.
	self beginPacket.! !

!AJPStream methodsFor: 'accessing' stamp: 'cs 6/15/2009 16:34'!
next
	"Answer the next object accessible by the receiver."
	
	^ self stream next! !

!AJPStream methodsFor: 'accessing' stamp: 'cs 6/14/2009 15:22'!
nextAttributeKey
	"Reads an encoded attribute code and returns a symbol"

	| code |
	code := self class attributeCode: self nextByte.
	^ code = #reqName "the name of the attribute is sent as string"
		ifTrue: [self nextString asSymbol]
		ifFalse: [code]! !

!AJPStream methodsFor: 'accessing' stamp: 'cs 6/14/2009 13:52'!
nextBoolean
	"Reads a boolean value from stream in AJP encoding"

	| result |
	result := self nextByte.
	result > 1 ifTrue: [self error: 'Invalid boolean value'].
	^ result = 1! !

!AJPStream methodsFor: 'accessing' stamp: 'cs 6/14/2009 15:34'!
nextByte
	"Reads a byte from stream"

	^ self next asInteger! !

!AJPStream methodsFor: 'accessing' stamp: 'cs 6/14/2009 13:51'!
nextHeaderKey
	"Reads a header key. This is either a string of a special header code"

	| start |
	start := self nextByte.
	^ start = 160 "If the key begins with 0xA0 parse special code"
		ifTrue: [self nextSpecialHeaderKey]
		ifFalse: [self nextString: (start bitShift: 8) + self nextByte]! !

!AJPStream methodsFor: 'accessing' stamp: 'cs 6/14/2009 15:50'!
nextInteger
	"Reads an integer value from stream in AJP encoding"

	| high | "Use temporal variable to force byte order"
	high := self nextByte bitShift: 8.
	^ high + self nextByte.! !

!AJPStream methodsFor: 'accessing' stamp: 'cs 6/14/2009 15:03'!
nextMethodCode
	"Reads an encoded method code"

	^ self class methodCode: self nextByte! !

!AJPStream methodsFor: 'accessing' stamp: 'cs 6/14/2009 15:06'!
nextPacketType
	"Reads an encoded packet type and returns the corresponding process method"

	^ self class packetType: self nextByte! !

!AJPStream methodsFor: 'accessing' stamp: 'cs 6/15/2009 17:19'!
nextPut: anObject 
	"Insert the argument, anObject, as the next object accessible by the 
	receiver. Answer anObject."
	
	^ self packetStream
		ifNil: [self stream nextPut: anObject]
		ifNotNil: [self packetStream nextPut: anObject]! !

!AJPStream methodsFor: 'accessing' stamp: 'cs 6/15/2009 20:42'!
nextPutBoolean: aBoolean
	"Writes the provided boolean to the underlying stream."

	aBoolean
		ifTrue: [self nextPutByte: 1]
		ifFalse: [self nextPutByte: 0].! !

!AJPStream methodsFor: 'accessing' stamp: 'cs 6/15/2009 18:01'!
nextPutByte: aNumber
	"Writes the provided byte to the underlying stream."
	
	self nextPut: (Character value: aNumber).! !

!AJPStream methodsFor: 'accessing' stamp: 'cs 6/15/2009 18:00'!
nextPutInteger: aSmallInteger
	"Writes a small integer to stream in AJP encoding"

	aSmallInteger < 0 | aSmallInteger >= 65536
		ifTrue: [self error: 'Invalid 16bit unsigned int'].
	self
		nextPutByte: (aSmallInteger bitShift: -8);
		nextPutByte: (aSmallInteger bitAnd: 255).! !

!AJPStream methodsFor: 'accessing' stamp: 'cs 6/15/2009 18:10'!
nextPutResponseHeader: headerString
	"Writes either the provided String or the special AJP code onto the underlying stream"

	(self class responseHeaderCode: headerString)
		ifNil: [self nextPutString: headerString]
		ifNotNilDo: [:code |
			self
				nextPutByte: 160; "Every response header code begins with 0xA0"
				nextPutByte: code].! !

!AJPStream methodsFor: 'accessing' stamp: 'cs 6/15/2009 18:29'!
nextPutString: aString
	"Writes a string to stream in AJP encoding"

	"A size of 2^16 - 1 indicates a null string"
	aString ifEmpty: [self nextPutInteger: 65535].
	self
		nextPutInteger: aString size;
		nextPutAll: aString;
		nextPutByte: 0.! !

!AJPStream methodsFor: 'accessing' stamp: 'cs 6/14/2009 13:53'!
nextSpecialHeaderKey
	"Reads the next header key encoded as special byte code"

	^ self class headerCode: self nextByte! !

!AJPStream methodsFor: 'accessing' stamp: 'cs 6/14/2009 13:46'!
nextString
	"Reads a string from stream in AJP encoding"

	| length |
	length := self nextInteger.
	length = 65535 ifTrue: [^ '']. "A size of 2^16 - 1 indicates a null string"
	^ self nextString: length! !

!AJPStream methodsFor: 'accessing' stamp: 'cs 6/15/2009 18:26'!
nextString: length
	"Reads a string of the specified length in AJP encoding"

	| aByteArray |
	aByteArray := ByteArray new: length.
	1 to: length do: [:i | aByteArray at: i put: self nextByte].
	self nextByte. "Reads the nullbyte at the end"
	^ aByteArray asString! !

!AJPStream methodsFor: 'accessing' stamp: 'cs 6/15/2009 18:36'!
packetStream
	"Returns the current packet output stream"

	^ packetStream! !

!AJPStream methodsFor: 'accessing' stamp: 'cs 6/15/2009 18:36'!
packetStream: anObject
	"Sets the current packet output stream"

	packetStream := anObject.! !

!AJPStream methodsFor: 'accessing' stamp: 'cs 6/15/2009 16:31'!
stream
	"Returns the underlying stream."

	^ stream! !

!AJPStream methodsFor: 'accessing' stamp: 'cs 6/15/2009 16:31'!
stream: anObject
	"Sets the underlying stream"
	
	stream := anObject.! !

TcpService subclass: #AJPService
	instanceVariableNames: 'plug'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KomAJPServer'!

!AJPService commentStamp: 'cs 6/15/2009 20:44' prior: 0!
An AJPService is an implementation of a AJP13 compatible servlet container. It can server many connections at once. Each of these connections is handling by the AJPConnection class.

AJPServer is the AJP13 equivalent of the HttpService of KomHttpServer.

Instance Variables
	plug:		<Object>

plug
	- the target of the http processing, e.g. a seaside application.!

!AJPService class methodsFor: 'preferences' stamp: 'cs 6/13/2009 21:22'!
dataTimeout
	"The number of seconds to wait for data to be received on a
	given socket."

	^ #'AJPService-dataTimeout' binding ifNil: [10]! !

!AJPService methodsFor: 'accessing' stamp: 'cs 6/13/2009 22:02'!
plug
	"A plug processes HttpRequests and answers HttpResponses"

	^ plug! !

!AJPService methodsFor: 'accessing' stamp: 'cs 6/13/2009 22:02'!
plug: anObject
	"A plug processes HttpRequests and answers HttpResponses"

	plug := anObject.! !

!AJPService methodsFor: 'conversation' stamp: 'cs 6/14/2009 15:28'!
serve: aSocket
	"Create a new AJPConnection that serves the socket"

	AJPConnection new
		stream: ((SocketStream on: aSocket)
			timeout: self class dataTimeout;
			autoFlush: false;
			yourself);
		target: self plug;
		run.! !
